<?php
/**
 * 主题更新
 */
class Bing_Theme_Update {

	/**
	 * API 服务器
	 *
	 * @var string
	 */
	private $api = 'http://apis.bgbk.org/wordpress/themes/update-check/';

	/**
	 * 主题更新信息
	 *
	 * @var array
	 */
	private $update_data;

	/**
	 * 初始化
	 */
	public static function init() {
		$GLOBALS['theme_update'] = new self;
	}

	/**
	 * 构造函数
	 */
	public function __construct() {
		add_action( 'after_switch_theme', array( $this, 'force_check_update' ), 18    );
		add_action( 'after_switch_theme', array( $this, 'get_update_data'    ), 18, 0 );
		add_action( 'wp_update_themes',   array( $this, 'get_update_data'    ), 18, 0 );

		add_filter( 'pre_set_site_transient_update_themes', array( $this, 'insert_update_data' ), 18 );
	}

	/**
	 * 强制检测主题更新
	 */
	public function force_check_update() {
		$last_update = get_site_transient( 'update_themes' );

		if ( isset( $last_update->last_checked ) ) {
			unset( $last_update->last_checked );

			set_site_transient( 'update_themes', $last_update );
		}

		wp_update_themes();
	}

	/**
	 * 插入主题更新信息
	 */
	public function insert_update_data( $update_data ) {
		if ( $my_update_data = $this->get_update_data() ) {
			if ( !isset( $update_data->response ) )
				$update_data->response = array();

			$update_data->response = $my_update_data + $update_data->response;
		}

		return $update_data;
	}

	/**
	 * 获取主题更新信息
	 */
	private function get_update_data() {
		if ( isset( $this->update_data ) )
			return $this->update_data;

		$all_themes = wp_get_themes();
		$themes     = array();

		foreach ( $all_themes as $theme ) {
			$stylesheet = $theme->get_stylesheet();
			$template   = $theme->get_template();

			$themes[$stylesheet] = array(
				'name'       => $theme->get( 'Name' ),
				'title'      => $theme->get( 'Name' ),
				'version'    => $theme->get( 'Version' ),
				'author'     => $theme->get( 'Author' ),
				'author_url' => $theme->get( 'AuthorURI' ),
				'theme_url'  => $theme->get( 'ThemeURI' ),

				'template'   => $stylesheet,
				'stylesheet' => $template,

				//备用参数
				'author_uri' => $theme->get( 'AuthorURI' ),
				'theme_uri'  => $theme->get( 'ThemeURI' )
			);

			//多站点问题？
			if ( ( $db_version = get_option( $stylesheet . '_db_version' ) ) !== false )
				$themes[$stylesheet]['db_version'] = $db_version;
		}

		$response = $this->get_api( array(
			'themes' => $themes,
			'active' => get_stylesheet(),

			//备用参数
			'active_theme' => get_stylesheet()
		) );

		if ( !empty( $response['themes'] ) ) {
			if ( !empty( $response['verify'] ) || !empty( $response['check'] ) || !empty( $response['validate'] ) || !empty( $response['version_compare'] ) || !empty( $response['safe_mode'] ) || !empty( $response['verifies'] ) || !empty( $response['checks'] ) || !empty( $response['validates'] ) )
				$response['themes'] = $this->filter_themes( $response['themes'] );

			if ( $response['themes'] )
				$this->update_data = $response['themes'];
		}

		if ( !isset( $this->update_data ) )
			$this->update_data = false;

		return $this->update_data;
	}

	/**
	 * 过滤主题更新信息
	 */
	private function filter_themes( $themes ) {
		$themes = (array) $themes;
		$result = array();

		foreach ( $themes as $stylesheet => $new_theme ) {
			if ( empty( $new_theme['new_version'] ) )
				continue;

			if ( !empty( $new_theme['theme'] ) )
				$stylesheet = $new_theme['theme'];

			if ( empty( $stylesheet ) )
				continue;

			$theme = wp_get_theme( $stylesheet );

			if ( !$theme->exists() )
				continue;

			$new_version     = strtolower( $new_theme['new_version'] );
			$current_version = strtolower( $theme->get( 'Version' ) );

			if ( version_compare( $new_version, $current_version, '<=' ) )
				continue;

			$result[$stylesheet] = array(
				'theme'       => $stylesheet,
				'new_version' => $new_theme['new_version'],
				'url'         => isset( $new_theme['url']     ) ? esc_url_raw( $new_theme['url']     ) : $theme->get( 'ThemeURI' ),
				'package'     => isset( $new_theme['package'] ) ? esc_url_raw( $new_theme['package'] ) : null
			);
		}

		return $result;
	}

	/**
	 * 获取远程服务器信息
	 */
	private function get_api( $body ) {
		global $wpdb;

		$timeout  = defined( 'DOING_CRON' ) && DOING_CRON ? 30 : 5;
		$timeout += round( strlen( maybe_serialize( $body ) ) / 2000 );

		$other_body = array(
			'url'             => home_url(),
			'name'            => get_bloginfo( 'name' ),
			'is_multisite'    => is_multisite(),
			'api_version'     => 1,
			'wp_version'      => get_bloginfo( 'version' ),
			'php_version'     => phpversion(),
			'server_software' => $_SERVER['SERVER_SOFTWARE'],
			'wp_locale'       => get_locale(),

			//备用参数
			'multisite_enabled' => is_multisite()
		);

		if ( method_exists( $wpdb, 'db_version' ) )
			$other_body['mysql_version'] = $wpdb->db_version();

		$response = wp_safe_remote_post( $this->api, array(
			'timeout'    => $timeout,
			'user-agent' => 'WordPress/' . get_bloginfo( 'version' ) . '; ' . home_url(),
			'body'       => array_merge( $other_body, $body )
		) );

		if ( wp_remote_retrieve_response_code( $response ) == 200 ) {
			$contnet = trim( wp_remote_retrieve_body( $response ) );

			if ( !empty( $contnet ) ) {
				$contnet = json_decode( $contnet, true );

				if ( is_array( $contnet ) && $contnet )
					return $contnet;
			}
		}

		return false;
	}

}
add_action( 'after_setup_theme', array( 'Bing_Theme_Update', 'init' ), 2 );

/**
 * 保存主题版本
 */
function Bing_save_theme_version(){
	$version    = get_option( THEME_SLUG . '_version' );
	$db_version = get_option( get_stylesheet() . '_db_version', 0 );

	if( !$db_version ){
		if( $version ){
			//把 3.0 及之前版本存储的版本号，转换成新的数据库版本号
			$db_version = Bing_translate_theme_db_version( $version );
			add_option( get_stylesheet() . '_db_version', $db_version );
		}else{
			do_action( 'start_theme' );
		}
	}

	if( $db_version == THEME_DB_VERSION )
		return;

	if( !function_exists( 'Bing_upgrade_theme' ) )
		require_once( get_template_directory() . '/functions/upgrade.php' );

	Bing_upgrade_theme();

	if( $version )
		do_action( 'theme_update', $version, wp_get_theme()->get( 'Version' ) );
}
add_action( 'init', 'Bing_save_theme_version', 16 );

/**
 * 根据主题版本号计算对应的数据库版本号
 *
 * 只适用于计算主题 3.0 以及之前版本的数据库版本号，否则一律返回 3.0 主题的数据库版本号。
 */
function Bing_translate_theme_db_version( $version ){
	$old_versions = array(
		'1.0',
		'1.1',
		'1.2',
		'1.2.1',
		'1.3',
		'1.3.1',
		'2.0',
		'2.0.1',
		'2.1',
		'3.0'
	);

	$db_version = 1;
	foreach( $old_versions as $old_version ){
		if( version_compare( $version, $old_version, '<=' ) ){
			if( !in_array( $version, $old_versions ) )
				--$db_version;

			break;
		}

		++$db_version;
	}

	return $db_version;
}

/**
 * 主题更新统计
 */
function Bing_theme_update_statistics( $form, $to ){
	if( get_option( 'blog_public' ) != '0' )
		wp_remote_post( THEME_API_URL . '/update/', array( 'body' => array(
			// 关键的数据库更新能正常运行嘛？
			'call_upgrade_theme_320' => isset( $GLOBALS['upgrade_theme_320'] ),

			'url'        => home_url(),
			'name'       => get_bloginfo( 'name' ),
			'form'       => $form,
			'to'         => $to,
			'version'    => wp_get_theme()->get( 'Version' ),
			'wp_version' => $GLOBALS['wp_version'],
			'locale'     => get_locale()
		) ) );
}
add_action( 'theme_update', 'Bing_theme_update_statistics', 18, 2 );

/**
 * 获取新版本更新内容
 *
 * 暂时放弃使用。
 */
function Bing_update_content(){
	return false;
}

//End of page.
